#include <avr/io.h>
#include <avr/pgmspace.h>

#include "spi.h"

// Initialise the device as an SPI master.
void spi_init(void) {
	// Ensure that SS is high.
	SPI_PORT |= _BV(SPI_SS_BIT);
	// /SS, MOSI and SCK need to be outputs in SPI master mode.
	SPI_DDR |= _BV(SPI_SS_BIT) | _BV(SPI_MOSI_BIT) | _BV(SPI_SCK_BIT);
	// Enable the SPI peripheral in master mode.
	SPCR = _BV(SPE) | _BV(MSTR);
	// Double speed mode.
	SPSR |= SPI2X;
}

// Exchanges data with the SPI slave.
uint8_t spi_transmit(uint8_t data) {
	SPDR = data;
	while (!(SPSR & _BV(SPIF)));
	return SPDR;
}

// Set the SPI clock speed to a defined maximum.
void spi_set_speed(uint32_t max_speed) {
	SPSR |= _BV(SPI2X);
	if (max_speed > F_CPU / 16UL) {
		SPCR &= (uint8_t)~_BV(SPR1);
	} else {
		SPCR |= _BV(SPR1);	
	}
}