#include <stdint.h>
#include <avr/io.h>
#include <avr/pgmspace.h>

#include "i2c.h"

// Writes a single byte to the VDU.
void vdu_write(uint8_t value) {
	
	// Send START.
	TWCR = _BV(TWINT) | _BV(TWSTA) | _BV(TWEN);
	while (!(TWCR & _BV(TWINT)));
	
	// Send slave address.
	TWDR = 0xEE & 0xFE;
	TWCR = _BV(TWINT) | _BV(TWEN);
	while (!(TWCR & _BV(TWINT)));
	
	// Send data
	TWDR = value;
	TWCR = _BV(TWINT) | _BV(TWEN);
	while (!(TWCR & _BV(TWINT)));
	
	// Send STOP.
	TWCR = _BV(TWINT) | _BV(TWEN) | _BV(TWSTO);

}

// Writes a NUL-terminated string to the VDU.
void vdu_write_string(char* s) {
	char c;
	while ((c = *s++)) {
		vdu_write((uint8_t)c);
	}
}

// Writes a NUL-terminated string stored in ROM to the VDU.
void vdu_write_string_P(const char* s) {
	char c;
	while ((c = pgm_read_byte(s++))) {
		vdu_write(c);
	}
}