#include <stdint.h>

#include <avr/io.h>

#include <util/delay.h>

#include "mcp23s08.h"
#include "z80.h"
#include "z80_memory.h"
#include "pins.h"

// Initialise the Z80 memory accessing routines.
void z80_memory_init(void) {
	// Enable the I/O expanders.
	mcp23s08_init();
	// Force to address 0000.
	z80_memory_set_address(0xFFFF);
	z80_memory_set_address(0x0000);
}

uint8_t current_address_l = 0x00;
uint8_t current_address_h = 0x00;

// Set the address on the Z80 bus.
void z80_memory_set_address(uint16_t address) {
	uint8_t address_c = address;
	if (current_address_l != address_c) {
		mcp23s08_write(0b00, MCP23S08_GPIO, current_address_l = address_c);
	}
	address_c = address >> 8;
	if (current_address_h != address_c) {
		mcp23s08_write(0b01, MCP23S08_GPIO, current_address_h = address_c);
	}
}

// Get the address from the Z80 bus.
uint16_t z80_memory_get_address(void) {
	return mcp23s08_read(0b00, MCP23S08_GPIO) | (mcp23s08_read(0b01, MCP23S08_GPIO) << 8);
}

// Drive the Z80 bus to take control of it.
void z80_memory_control_address(void) {
	mcp23s08_write(0b00, MCP23S08_IODIR, 0x00);
	mcp23s08_write(0b01, MCP23S08_IODIR, 0x00);
}

// Release the address bus to its high impedence state.
void z80_memory_release_address(void) {
	mcp23s08_write(0b00, MCP23S08_IODIR, 0xFF);
	mcp23s08_write(0b01, MCP23S08_IODIR, 0xFF);	
}


// Read a byte from the Z80 memory.
uint8_t z80_memory_read(void) {
	PIN_LOW(Z80_RD);
	PIN_LOW(Z80_MREQ);
	_delay_us(1);
	uint8_t value = Z80_DATA_PIN;
	PIN_HIGH(Z80_MREQ);
	PIN_HIGH(Z80_RD);
	return value;
}

// Write a byte to the Z80 memory.
void z80_memory_write(uint8_t value) {
	Z80_DATA_PORT = value;
	Z80_DATA_DDR = 0xFF;
	PIN_LOW(Z80_WR);
	PIN_LOW(Z80_MREQ);
	_delay_us(1);
	PIN_HIGH(Z80_MREQ);
	PIN_HIGH(Z80_WR);
	Z80_DATA_DDR = 0x00;
	Z80_DATA_PORT = 0x00;
}